"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildModelFieldObject = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const typescript_1 = require("typescript");
/**
 * builds modelFields object which is used to validate, onSubmit, onSuccess/onError
 * the nameOverrides will swap in a different expression instead of the name of the state when building the object
 *
 * ex.  [name, content, updatedAt]
 *
 * const modelFields = {
 *   name,
 *   content,
 *   updatedAt
 * };
 * @param fieldConfigs
 * @returns
 */
const buildModelFieldObject = (shouldBeConst, fieldConfigs = {}, models = {}, nameOverrides = {}, isRenderingGraphQL = false) => {
    const fieldSet = new Set();
    const fields = Object.keys(fieldConfigs).reduce((acc, value) => {
        var _a;
        const fieldName = value.split('.')[0];
        const { sanitizedFieldName, relationship } = fieldConfigs[value];
        const renderedFieldName = sanitizedFieldName || fieldName;
        if (!fieldSet.has(renderedFieldName)) {
            let assignments = [
                typescript_1.factory.createShorthandPropertyAssignment(typescript_1.factory.createIdentifier(fieldName), undefined),
            ];
            if (nameOverrides[fieldName]) {
                assignments = [nameOverrides[fieldName]];
            }
            else if (isRenderingGraphQL &&
                typeof fieldConfigs[value].dataType === 'object' &&
                relationship &&
                (relationship.type === 'BELONGS_TO' || relationship.type === 'HAS_ONE')) {
                assignments =
                    ((_a = relationship.associatedFields) === null || _a === void 0 ? void 0 : _a.map((associatedField, index) => {
                        return typescript_1.factory.createPropertyAssignment(typescript_1.factory.createStringLiteral(associatedField), typescript_1.factory.createPropertyAccessChain(typescript_1.factory.createIdentifier(fieldName), undefined, models[relationship.relatedModelName].primaryKeys[index]));
                    })) || [];
            }
            else if (sanitizedFieldName) {
                // if overrides present, ignore sanitizedFieldName
                assignments = [
                    typescript_1.factory.createPropertyAssignment(typescript_1.factory.createStringLiteral(fieldName), typescript_1.factory.createIdentifier(sanitizedFieldName)),
                ];
            }
            acc.push(...assignments);
            fieldSet.add(renderedFieldName);
        }
        return acc;
    }, []);
    return typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
        typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('modelFields'), undefined, undefined, typescript_1.factory.createObjectLiteralExpression(fields, true)),
    ], shouldBeConst ? typescript_1.NodeFlags.Const : typescript_1.NodeFlags.Let));
};
exports.buildModelFieldObject = buildModelFieldObject;
//# sourceMappingURL=model-fields.js.map